#include "Keyhole.h"

double foo = 0.0;
String bar = "look at these escaped characters: \" \x08 \\ \t\r\n";

void setup()
{
  Serial.begin(9600);
}

void loop()
{
	// The main business of your sketch goes here...
  // ...
  // A block of Keyhole commands then provides a lightweight
  // interface for examining and controlling that business.
  
  // First, the KEYHOLE macro declares a `static Keyhole`
  // instance (you can also feel free to put this declaration
  // outside the `loop()` function, if you want the instance to
  // be global):
  
  KEYHOLE keyhole(Serial);
  
  if(keyhole.begin()) // on most loops, this will return false, so
  {                   // very little processing will need to be done
    
    // With the `.command()` method, you can respond to serial
    // commands that are not bound to particular variables:

    if(keyhole.command("marco!"))
      Serial.println("polo!");
    
    // ...but the principal value of Keyhole is in the ease
    // with which it exposes *variables* via the serial port:
    
    keyhole.variable("foo", foo);
    
    // Now you can query the value just by entering the name
    // foo at the Serial Monitor prompt (or sending "foo\n"
    // from some other application - all serial commands must
    // end with a newline or semicolon). You can also *change*
    // the value, e.g with the serial command:  foo=123
    
    // There are three possible built-in behaviors when you
    // assign a value in this way. The default is called
    // `VARIABLE_SILENT`, but you can experiment by passing
    // either `VARIABLE_VERBOSE` or `VARIABLE_READ_ONLY` as a
    // third argument to the `.variable()` method.
    
    // You can also customize the behavior: the `.variable()`
    // method returns `true` when the user uses the serial port
    // to assign a value to the variable in question, so you
    // can opt to perform additional actions in that case:
    
    if(keyhole.variable("bar", bar))
      Serial.println("thank you for assigning a value to bar");
    
    // Note that the variable `bar` is a `String`, so the
    // serial command you use for assignment will need to
    // include quotes:  bar = "abcdefghi"
    
    // If quotes or backslashes appear as part of the string
    // content itself, you must backslash-escape them. Inside
    // the quotes you can also express non-printable bytes as
    // hex escapes, like \x08 (this is limited to two hex
    // digits - same rules as Python). Old favorites \r \n \t \0
    // are also recognized.
    
    // Keyhole supports all the various types of integers,
    // floats and doubles, and Strings.  Numeric values can
    // be assigned in decimal as usual, or in hex with a 0x
    // prefix, or in binary with a 0b prefix.

    // You can query all exposed variables at once by sending
    // the "?" command.  You can also opt to receive regular
    // automatic reports, by setting the `keyhole.autoSeconds`
    // member to a value greater than 0.0.  In fact, why don't
    // we expose that as a variable, through the keyhole:
    
    keyhole.variable("response_period_sec",  keyhole.autoSeconds);
    
    // Try it, by sending the command:  response_period_sec=2
        
    keyhole.end(); // must call this if `.begin()` returned `true`
    
    // Finally, you can examine the impact Keyhole has on your
    // sketch's timing by passing `micros()` into `keyhole.begin()`
    // and then getting the number of microseconds elapsed with
    // `keyhole.elapsed()`, either here directly after the `.end()`
    // call (to measure the impact on loops in which a command was
    // received)...
  }
  // ... or in an `else` clause here, to measure the more-typical
  // loops when no command was received and so `.begin()` returned
  // `false`.
  
  // All serial messages sent by Keyhole are single lines
  // ending in a newline character,  are valid JSON literals,
  // and can be turned into dicts in Python by saying::
  // 
  //     import ast;  d = ast.literal_eval(msg.decode())
}
